package CalculiX;

public class Main {

  // Set up the factory.
  static ExpressionFactory factory = new ExpressionFactory ();

  // The main method.

  public static void main (String[] argv) throws Exception
  {
    // Performs the first j tests
    // Change j into whichever number of tests you want to perform
    int j = 0;

    if (argv.length>0) j = java.lang.Integer.parseInt(argv[0]);
    for(int i=0;i<j;i++)
	    interpretAndValidate(input[i],output[i]);
	
    System.out.println("All ok, testing from test0 to test"+(j-1));
  }


  // This method initializes and runs the parser.
  // It requires a lexer and uses the factory.
  // It returns an expression.

  static Expression parse (MyLexer lexer) throws Exception
  {
    // Set up the parser.
    MyParser parser = new MyParser (lexer,factory);
    // Allow the parser to access the expression factory.
    // Run the parser.
    // The method parse() produces a Symbol, whose semantic value is
    // an abstract syntax tree for an expression.
    return (Expression) parser.parse();
  }

  // This method runs the parser and the interpreter.
  // It requires a Reader.
  // It produces a Value.

  static Value parseAndInterpret (String text) throws Exception
  {
    // Read from the supplied string.
    java.io.Reader reader = new java.io.StringReader (text);
    // Set up the lexer.
    MyLexer lexer = new MyLexer (reader);
    // Set up and run the parser.
    Expression e = parse (lexer);
    System.out.println("Parsed \""+text+"\" as "+e);
    // Interpret the expression.
    Value v = e.eval(Environment.empty);
    // Return the result.
    return v;
  }

  // A test method.
  // It reads from a string.
  // It parses an expression, evaluates it, and compares its value with
  // the expected value (which must be an integer value).

  static void interpretAndValidate (String text, int expectedResult) throws Exception
  {
    // Parse and interpret.
    Value v = parseAndInterpret (text);

    // Make sure that the actual result is an integer value,
    // and compare with the expected result.
    System.out.println("Evaluated as "+v.asInteger()+ ", expecting "+ expectedResult );
    assert expectedResult == v.asInteger();
  }
  
  static void interpretAndValidate (String text, boolean expectedResult) throws Exception
  {
    // Parse and interpret.
    Value v = parseAndInterpret (text);

    // Make sure that the actual result is an integer value,
    // and compare with the expected result.
    System.out.println("Evaluated as "+v.asBoolean()+ ", expecting "+ expectedResult );
    assert expectedResult == v.asBoolean();
  }


  // Individual tests.

  static String[] input = 
  {"0",
   "7",
   // "#",  // must raise a syntax error
   // "42 x",  // must raise a syntax error

   "2+3",
   "2-3",
   "3-2",
   "3-2-1",

   "3-(2-1)",

   "6+(3*8)",
   "8/4/2",
   "1-2+3",
   "8/2*4",
   "1+2*3",
   "1+3/2",
   "2*3-1",
   "16/4-1",
   "2 + 3*5 - 30/2 - 2",
   "(5 - 2) * 2 - 24 / (2 + 2)",

   "if true then 0 else 1",
   "if (if false then true else false) then 8 else 1",
   "if 5<4 then 3 else 2",
   "if 1+1<1 || 4<4*4 then 3 else 4",

   // "let x = 2 * 21 in x",
   // "let x = 2 in let y = 21 in x * y",
   // "let x = 2 in let x = 21 in x * x",
   // "let x = 2 in let y = x + 19 in x * y",
   // "let x = 2 in let x = x + 19 in x * x",
   // "(let x = 2 in x + x) * (let x = 3 in x + x)",
   // "for x = 0 to 10 sum x",
   // "1 + (for x = 0 to 10 sum x * x)",

   "1+42++3",
   "(42))",
   "((42)",
   "true + true",
   "if 0 then 1 else 2",

   // "x",
   // "let x = x + 1 in x",
   // "(let x = 2 in x) + x"
};

  static int[] output = 
  {0,
   7,
   // must raise syntax error
   // must raise syntax error
   5,
   -1,
   1,
   0,

   2,

   30,
   1,
   2,
   16,
   7,
   2,
   5,
   3,
   0, 
   0,
     
   0,
   1,
   2,
   3,

   // 42,
   // 42,
   // 441,
   // 42,
   // 441,
   // 24,
   // 55,
   // 386,
  };


}
